"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RulesAction = void 0;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const WorkspaceUtil_1 = require("../utils/WorkspaceUtil");
const LogWriter_1 = require("../writers/LogWriter");
const LogEventListener_1 = require("../listeners/LogEventListener");
class RulesAction {
    dependencies;
    constructor(dependencies) {
        this.dependencies = dependencies;
    }
    async execute(input) {
        const config = this.dependencies.configFactory.create(input['config-file']);
        const logWriter = await LogWriter_1.LogFileWriter.fromConfig(config);
        // We always add a Logger Listener to the appropriate listeners list, because we should Always Be Logging.
        this.dependencies.logEventListeners.push(new LogEventListener_1.LogEventLogger(logWriter));
        const core = new code_analyzer_core_1.CodeAnalyzer(config);
        // LogEventListeners should start listening as soon as the Core is instantiated, since Core can start emitting
        // events they listen for basically immediately.
        this.dependencies.logEventListeners.forEach(listener => listener.listen(core));
        const enginePlugins = this.dependencies.pluginsFactory.create();
        const enginePluginModules = config.getCustomEnginePluginModules();
        const addEnginePromises = [
            ...enginePlugins.map(enginePlugin => core.addEnginePlugin(enginePlugin)),
            ...enginePluginModules.map(pluginModule => core.dynamicallyAddEnginePlugin(pluginModule))
        ];
        await Promise.all(addEnginePromises);
        const selectOptions = input.workspace
            ? { workspace: await (0, WorkspaceUtil_1.createWorkspace)(core, input.workspace) }
            : undefined;
        // EngineProgressListeners should start listening right before we call Core's `.selectRules()` method, since
        // that's when progress events can start being emitted.
        this.dependencies.progressListeners.forEach(listener => listener.listen(core));
        const ruleSelection = await core.selectRules(input["rule-selector"], selectOptions);
        // After Core is done running, the listeners need to be told to stop, since some of them have persistent UI elements
        // or file handlers that must be gracefully ended.
        this.dependencies.progressListeners.forEach(listener => listener.stopListening());
        this.dependencies.logEventListeners.forEach(listener => listener.stopListening());
        const rules = core.getEngineNames().flatMap(name => ruleSelection.getRulesFor(name));
        this.dependencies.viewer.view(rules);
    }
    static createAction(dependencies) {
        return new RulesAction(dependencies);
    }
}
exports.RulesAction = RulesAction;
//# sourceMappingURL=RulesAction.js.map