"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResultsTableViewer = exports.ResultsDetailViewer = void 0;
exports.findLongestCommonParentFolderOf = findLongestCommonParentFolderOf;
const code_analyzer_core_1 = require("@salesforce/code-analyzer-core");
const StylingUtil_1 = require("../utils/StylingUtil");
const messages_1 = require("../messages");
const node_path_1 = __importDefault(require("node:path"));
class AbstractResultsViewer {
    display;
    constructor(display) {
        this.display = display;
    }
    view(results) {
        if (results.getViolationCount() === 0) {
            this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.found-no-results'));
        }
        else {
            this._view(results);
        }
    }
    countUniqueFiles(violations) {
        const fileSet = new Set();
        violations.forEach(v => {
            const primaryLocation = v.getCodeLocations()[v.getPrimaryLocationIndex()];
            const file = primaryLocation.getFile();
            if (file) {
                fileSet.add(file);
            }
        });
        return fileSet.size;
    }
}
class ResultsDetailViewer extends AbstractResultsViewer {
    _view(results) {
        const violations = sortViolations(results.getViolations());
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.found-results', [
            violations.length, this.countUniqueFiles(violations)
        ]));
        this.displayDetails(violations);
        this.display.displayLog('\n');
        this.displayBreakdown(results);
    }
    displayDetails(violations) {
        const styledViolations = violations
            .map((violation, idx) => this.styleViolation(violation, idx));
        this.display.displayLog(styledViolations.join('\n\n'));
    }
    styleViolation(violation, idx) {
        const rule = violation.getRule();
        const sev = rule.getSeverityLevel();
        const primaryLocation = violation.getCodeLocations()[violation.getPrimaryLocationIndex()];
        const header = (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.violation-header', [idx + 1, rule.getName()]);
        const body = {
            severity: `${sev.valueOf()} (${code_analyzer_core_1.SeverityLevel[sev]})`,
            engine: rule.getEngineName(),
            message: violation.getMessage(),
            location: `${primaryLocation.getFile()}:${primaryLocation.getStartLine()}:${primaryLocation.getStartColumn()}`,
            resources: violation.getResourceUrls().join(',')
        };
        const keys = ['severity', 'engine', 'message', 'location', 'resources'];
        return (0, StylingUtil_1.toStyledHeaderAndBody)(header, body, keys);
    }
    displayBreakdown(results) {
        this.display.displayLog((0, StylingUtil_1.toStyledHeader)((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.breakdown.header')));
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.breakdown.total', [results.getViolationCount()]));
        for (const sev of Object.values(code_analyzer_core_1.SeverityLevel)) {
            // Some of the keys will be numbers, since the enum is numerical. Skip those.
            if (typeof sev !== 'string') {
                continue;
            }
            const sevCount = results.getViolationCountOfSeverity(code_analyzer_core_1.SeverityLevel[sev]);
            if (sevCount > 0) {
                this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.detail.breakdown.item', [sevCount, sev]));
            }
        }
    }
}
exports.ResultsDetailViewer = ResultsDetailViewer;
const TABLE_COLUMNS = {
    num: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.num-column'),
    },
    severity: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.severity-column')
    },
    rule: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.rule-column')
    },
    location: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.location-column')
    },
    message: {
        header: (0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.message-column')
    }
};
class ResultsTableViewer extends AbstractResultsViewer {
    _view(results) {
        const violations = sortViolations(results.getViolations());
        const parentFolder = findLongestCommonParentFolderOf(violations.map(v => getPrimaryLocation(v).getFile()).filter(f => f !== undefined));
        const resultRows = violations.map((v, idx) => {
            const severity = v.getRule().getSeverityLevel();
            const primaryLocation = getPrimaryLocation(v);
            const relativeFile = primaryLocation.getFile() ?
                node_path_1.default.relative(parentFolder, primaryLocation.getFile()) : null;
            return {
                num: idx + 1,
                location: `${relativeFile}:${primaryLocation.getStartLine()}:${primaryLocation.getStartColumn()}`,
                rule: `${v.getRule().getEngineName()}:${v.getRule().getName()}`,
                severity: `${severity.valueOf()} (${code_analyzer_core_1.SeverityLevel[severity]})`,
                message: v.getMessage()
            };
        });
        this.display.displayLog((0, messages_1.getMessage)(messages_1.BundleName.ResultsViewer, 'summary.table.found-results', [
            violations.length, this.countUniqueFiles(violations), parentFolder
        ]));
        this.display.displayTable(resultRows, TABLE_COLUMNS);
    }
}
exports.ResultsTableViewer = ResultsTableViewer;
function sortViolations(violations) {
    return violations.toSorted((v1, v2) => {
        // First compare severities.
        const v1Sev = v1.getRule().getSeverityLevel();
        const v2Sev = v2.getRule().getSeverityLevel();
        if (v1Sev !== v2Sev) {
            return v1Sev - v2Sev;
        }
        // Next, compare file names.
        const v1PrimaryLocation = v1.getCodeLocations()[v1.getPrimaryLocationIndex()];
        const v2PrimaryLocation = v2.getCodeLocations()[v2.getPrimaryLocationIndex()];
        const v1File = v1PrimaryLocation.getFile() || '';
        const v2File = v2PrimaryLocation.getFile() || '';
        if (v1File !== v2File) {
            return v1File.localeCompare(v2File);
        }
        // Next, compare start lines.
        const v1StartLine = v1PrimaryLocation.getStartLine() || 0;
        const v2StartLine = v2PrimaryLocation.getStartLine() || 0;
        if (v1StartLine !== v2StartLine) {
            return v1StartLine - v2StartLine;
        }
        // Next, compare start columns.
        const v1StartColumn = v1PrimaryLocation.getStartColumn() || 0;
        const v2StartColumn = v2PrimaryLocation.getStartColumn() || 0;
        return v1StartColumn - v2StartColumn;
    });
}
// TODO: We should update core module to have this function directly on the Violation object and then remove this helper
function getPrimaryLocation(violation) {
    return violation.getCodeLocations()[violation.getPrimaryLocationIndex()];
}
/**
 * Returns the longest comment parent folder of the file paths provided
 * Note that this function assumes that all the paths are indeed files and not folders
 */
function findLongestCommonParentFolderOf(filePaths) {
    const roots = filePaths.map(filePath => node_path_1.default.parse(filePath).root);
    const commonRoot = (new Set(roots)).size === 1 ? roots[0] : '';
    if (!commonRoot) {
        return '';
    }
    const commonFolders = [];
    let depth = 0;
    const explodedPaths = filePaths.map(file => node_path_1.default.dirname(file).slice(commonRoot.length).split(node_path_1.default.sep).filter(v => v.length > 0));
    while (explodedPaths.every(folders => folders.length > depth)) {
        const currFolder = explodedPaths[0][depth];
        if (explodedPaths.some(folders => folders[depth] !== currFolder)) {
            break;
        }
        commonFolders.push(currFolder);
        depth++;
    }
    return node_path_1.default.join(commonRoot, ...commonFolders);
}
//# sourceMappingURL=ResultsViewer.js.map